@@ -1,6 +1,15 @@
 `QAIC` <-
 function(object, ..., chat) {
 	#chat <- summary(gm)$dispersion
+
+	`getQAIC` <- function(object, chat) {
+		mLogLik <- logLik(object)
+		N <- length(residuals(object))
+		k <- attr(mLogLik, "df") + 1
+		ret <- (deviance(object) / chat) + 2 * k
+		return (ret)
+	}
+
 	if(length(list(...))) {
 		object <- list(object, ...)
 		val <- data.frame(QAIC=sapply(object, getQAIC, chat = chat))
@@ -12,4 +21,3 @@ function(object, ..., chat) {
 		return(getQAIC(object, chat = chat))
 	}
 }
-
