@@ -21,11 +21,16 @@ function(x) {
 
 # test for marginality constraints
 `formulaAllowed` <-
-function(frm) {
+function(frm, except=NULL) {
+	if(isTRUE(except)) return(TRUE)
 	factors <- attr(terms(frm), "factors")
+	if(length(factors) == 0) return(TRUE)
+	if(is.character(except))
+		factors <- factors[!(rownames(factors) %in% except), ]
 	return(all(factors < 2))
 }
 
+
 # Calculate Akaike weights
 `Weights` <-
 function(aic, ...) {
@@ -33,3 +38,11 @@ function(aic, ...) {
 	weight <- exp(-delta / 2) / sum(exp(-delta / 2))
 	return (weight)
 }
+
+
+#sorts alphabetically interaction components in model term names
+`fixCoefNames` <-
+function(x) {
+	if(!is.character(x)) return(x)
+	return(sapply(lapply(strsplit(x, ":"), sort), paste, collapse=":"))
+}
